#!/usr/bin/env python
#
# Lightsaber rotoscoping scripts
# Splits the mosaic into individual images
# Recombines frames and audio file into a movie
#
# Matthew Beckler - matthew at mbeckler dot org
# http://www.mbeckler.org/lightsabers/
# Version 1.01 - May 17, 2011

import sys, os, shutil
from subprocess import *
from math import ceil, sqrt
from PIL import Image

if len(sys.argv) < 2:
    print "Usage: %s workingDirectory" % sys.argv[0]
    print "Provide the name of the directory created by the first script."
    sys.exit(1)

print "Parsing command line options ..."
basename = sys.argv[1]
if not os.path.exists(basename):
    print "Directory '%s' doesn't exist, try again" % basename
    sys.exit(1)
os.chdir(basename)

print "Reading project_info file ..."
proj_filename = "project_info"
if not os.path.exists(proj_filename):
    print "The project_info file doesn't exist within '%s'" % basename
    sys.exit(1)
try:
    proj_file = open(proj_filename, "r")
    basename = proj_file.readline().split()[0]
    width = int(proj_file.readline().split()[0])
    height = int(proj_file.readline().split()[0])
    num_frames = int(proj_file.readline().split()[0])
    buffer_size = int(proj_file.readline().split()[0])
    proj_file.close()
except:
    print "There was an error while reading the project_info file!"
    raise

print "Creating output directory ..."
output_dir = "finished_frames"
if os.path.exists(output_dir):
    shutil.rmtree(output_dir)
os.mkdir(output_dir)

print "Splitting mosaic ..."

# we use the square root to make the mosaic as square as possible
num_cols = int(ceil(sqrt(num_frames)))
num_rows = int(ceil(num_frames / (1.0 * num_cols)))
print "Number of rows: %d" % num_rows
print "Number of columns: %d" % num_cols

print "Loading mosaic file ..."
mosaic = Image.open("mosaic.png")

sys.stdout.write("Splitting images ")
sys.stdout.flush()
i = 1
for y in range(0, num_rows):
    for x in range(0, num_cols):
        if i <= num_frames:
            cut_x = (x * ((2 * buffer_size) + width)) + buffer_size
            cut_y = (y * ((2 * buffer_size) + height)) + buffer_size
            img_filename = os.path.join(output_dir, "%08d.png" % i)
            img = mosaic.copy().crop((cut_x, cut_y, cut_x + width, cut_y + height))
            img.save(img_filename)
            sys.stdout.write(".")
            sys.stdout.flush()
            i = i + 1
        else:
            break
print " done"

print "Converting finished frames into movie ..."
os.chdir(output_dir)
framerate = 25
output_filename = basename + "_finished.avi"
p = Popen(["mencoder", "mf://*.png", "-mf", "fps=%d:type=png" % framerate, "-ovc", "lavc", "-lavcopts", "vcodec=mpeg4", "-audiofile", "../audio.wav", "-oac", "mp3lame", "-o", output_filename], stdout=PIPE, stderr=PIPE)
os.waitpid(p.pid, 0)
os.chdir(os.pardir)
os.chdir(os.pardir)

shutil.copy(os.path.join(basename, output_dir, output_filename), os.curdir)

print "Cleaning up ..."
shutil.rmtree(os.path.join(basename, output_dir))

print "All finished!"
